/*
Copyright (C) 2020 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2017 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#pragma once

#include <vector>

#include "../Tools/GUI/Dialog/DialogBase.h"
#include "../Tools/GUI/ToolBar/Toolbar.h"

#define CDialogDesktopIconsPositions_APPLICATION_TITLE               TEXT("Desktop Icons Positions")

#define CDialogDesktopIconsPositions_PROJECT_NAME                    TEXT("DesktopIconsPositions")
#define CDialogDesktopIconsPositions_WEB_ROOT_LINK                   TEXT("http://jacquelin.potier.free.fr/")
#define CDialogDesktopIconsPositions_WEB_PROJECT_LINK                CDialogDesktopIconsPositions_WEB_ROOT_LINK CDialogDesktopIconsPositions_PROJECT_NAME TEXT("/")
#define CDialogDesktopIconsPositions_CONFIGURATION_FILE_NAME         CDialogDesktopIconsPositions_PROJECT_NAME TEXT(".ini")
// #define CDialogDesktopIconsPositions_HELP_FILE                       CDialogDesktopIconsPositions_PROJECT_NAME TEXT(".chm")

// defines for update checking
#define CDialogDesktopIconsPositions_PAD_Url                     CDialogDesktopIconsPositions_WEB_PROJECT_LINK CDialogDesktopIconsPositions_PROJECT_NAME TEXT(".xml")
#define CDialogDesktopIconsPositions_DownloadLink                CDialogDesktopIconsPositions_WEB_ROOT_LINK TEXT("exe/") CDialogDesktopIconsPositions_PROJECT_NAME TEXT("_bin.zip")
#define CDialogDesktopIconsPositions_VersionSignificantDigits    3

class CDialogDesktopIconsPositions : public CDialogBase//, public CToolbarEventsBase
{
protected:
    // CDialogBase
    virtual BOOL OnInit();
    virtual BOOL OnClose();
    virtual void OnCommand(WPARAM wParam,LPARAM lParam);
    virtual BOOL OnNotify(WPARAM wParam,LPARAM lParam);

    // CDialogDesktopIconsPositions
    BOOL bUsePrimaryDisplayMonitorOnly;

    CRITICAL_SECTION RealDesktopItemsInfosCriticalSection;

    typedef struct tagRealDesktopItemInfos
    {
        int ItemIndex;
        POINT Position;
        TCHAR Name[MAX_PATH];
    }REAL_DESKTOP_ITEM_INFOS;

    std::vector <REAL_DESKTOP_ITEM_INFOS> RealDesktopItemsInfos;

    CToolbar* pToolbar;
    HWND hDesktopListView;
    POINT PointDefaultPosition;
    int IconWidth;
    int IconHeight;

    HWND GetDesktopListView();
    BOOL GetDesktopItemsPos();
    BOOL OnSave();
    BOOL OnRestore();
    void CheckForUpdate();
    void BugReport();
    void GetIniFilePath(TCHAR* IniFilePath,DWORD IniFilePathMaxSize);
    static BOOL PointBelongsToVisiblePartOfMonitor(POINT* pPoint);
public:
    CDialogDesktopIconsPositions();
    ~CDialogDesktopIconsPositions();
    BOOL Restore(TCHAR* AbsoluteOrRelativePath,BOOL bMsgBoxOnSuccess);
    BOOL Save(TCHAR* AbsoluteOrRelativePath,BOOL bMsgBoxOnSuccess);
};