/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manages the about dialog
//-----------------------------------------------------------------------------

#include "about.h"

#include "resource.h"
#include "DialogDesktopIconsPositions.h" // for project name and web link

#include "../Tools/Version/Version.h"
#include "../Tools/File/StdFileOperations.h"
#include "../Tools/Donations/Donations.h"

#define ID_ABOUT_SYSLINK 10001

//#define ABOUT_SYSLINK_TEXT TEXT("For more information, see the <A ID=\"IdHomePage\">project homepage</A>\r\nor <A ID=\"IdUserDoc\">user manual</A>\r\nAuthor : Jacquelin POTIER\r\n               <A ID=\"IdMailto\">jacquelin.potier@free.fr</A>")
#define ABOUT_SYSLINK_TEXT TEXT("For more information, see the <A ID=\"IdHomePage\">project homepage</A>\r\nAuthor : Jacquelin POTIER\r\n               <A ID=\"IdMailto\">jacquelin.potier@free.fr</A>")
#define ABOUT_SYSLINK_PROJECT_NAME CDialogDesktopIconsPositions_PROJECT_NAME
#define ABOUT_SYSLINK_PROJECT_LINK CDialogDesktopIconsPositions_WEB_PROJECT_LINK
//#define ABOUT_SYSLINK_PROJECT_USER_DOC_LINK ABOUT_SYSLINK_PROJECT_LINK TEXT("index.php#documentation_user")
#define ABOUT_SYSLINK_EMAIL_LINK TEXT("mailto:jacquelin.potier@free.fr")

#define ABOUT_SYSLINK_XPOS 100
#define ABOUT_SYSLINK_YPOS 80
#define ABOUT_SYSLINK_WIDTH 350
#define ABOUT_SYSLINK_HEIGHT 65

#define ABOUT_TITLE_FONT_HEIGHT 18
#define ABOUT_LINK_FONT_HEIGHT 16

CAbout::CAbout()
{
    this->pSysLinkAbout=NULL;
    this->hFontAboutTitle=NULL;
    this->hFontAboutLink=NULL;
}
CAbout::~CAbout()
{

}

BOOL CAbout::OnInit()
{
    TCHAR NameAndVersion[MAX_PATH];
    TCHAR AppFileName[MAX_PATH];
    TCHAR PrettyVersion[MAX_PATH];

    CStdFileOperations::GetAppPath(AppFileName,MAX_PATH);
    CVersion VersionInfo;
    if (VersionInfo.Read(AppFileName))
    {
        CVersion::GetPrettyVersion(VersionInfo.ProductVersion,3,PrettyVersion,MAX_PATH);
        _sntprintf(NameAndVersion,MAX_PATH,_T("%s v%s"),
                    VersionInfo.ProductName,
                    PrettyVersion
                    );
    }
    else
        *NameAndVersion=0;

    // set Font for Title
    this->hFontAboutTitle=::CreateFont(ABOUT_TITLE_FONT_HEIGHT,0,0,0,FW_BOLD,0,0,0,
                                    ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH,
                                    _T("MS Shell Dlg"));

    SendMessage(this->GetDlgItem(IDC_STATIC_ABOUT_TITLE),WM_SETFONT,(WPARAM)hFontAboutTitle,0);

    // set title (name and version)
    this->SetDlgItemTextT(IDC_STATIC_ABOUT_TITLE,NameAndVersion);

    // show syslink
    this->pSysLinkAbout=new CSysLink(this->GetControlHandle(),ID_ABOUT_SYSLINK,
                                    ABOUT_SYSLINK_TEXT,
                                    ABOUT_SYSLINK_XPOS,
                                    ABOUT_SYSLINK_YPOS,
                                    ABOUT_SYSLINK_WIDTH,
                                    ABOUT_SYSLINK_HEIGHT);

    // set Font for syslink
    this->hFontAboutLink=CreateFont(ABOUT_LINK_FONT_HEIGHT,0,0,0,FW_NORMAL,0,0,0,
                                    ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH,
                                    _T("MS Shell Dlg"));
    SendMessage(this->pSysLinkAbout->GetControlHandle(),WM_SETFONT,(WPARAM)this->hFontAboutLink,0);

    this->SendDlgMessage(IDC_STATIC_LICENSE,WM_SETFONT,(WPARAM)this->hFontAboutLink,0);
    this->SendDlgMessage(IDC_STATIC_MAKE_DONATION,WM_SETFONT,(WPARAM)this->hFontAboutLink,0);

    this->SetDlgButtonIcon(IDC_BUTTON_DONATION,IDI_ICON_DONATION,16,16);
    return TRUE;
}
BOOL CAbout::OnClose()
{
    if (this->hFontAboutTitle)
    {
        DeleteFont(this->hFontAboutTitle);
        this->hFontAboutTitle = NULL;
    }

    if (this->hFontAboutLink)
    {
        DeleteFont(this->hFontAboutLink);
        this->hFontAboutLink = NULL;
    }

    DeleteAndCleanIfNotNull(this->pSysLinkAbout);

    return TRUE;
}
void CAbout::OnCommand(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    switch (LOWORD(wParam))
    {
    case IDC_BUTTON_DONATION:
        CDonations::MakePaypalDonation(this->GetControlHandle(),ABOUT_SYSLINK_PROJECT_NAME,ABOUT_SYSLINK_PROJECT_LINK);
        break;
    }
}
BOOL CAbout::OnNotify(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(wParam);

    WCHAR IDName[48];
    if (this->pSysLinkAbout)
    {
        if (this->pSysLinkAbout->Notify(lParam,(LPWSTR)&IDName))
        {
            if (wcscmp(IDName,L"IdHomePage")==0)
            {
                pSysLinkAbout->OpenLink(ABOUT_SYSLINK_PROJECT_LINK);
            }
            //else if (wcscmp(IDName,L"IdUserDoc")==0)
            //{
            //    pSysLinkAbout->OpenLink(ABOUT_SYSLINK_PROJECT_USER_DOC_LINK);
            //}
            else // IdMailTo
            {
                pSysLinkAbout->OpenLink(ABOUT_SYSLINK_EMAIL_LINK);
            }

            return TRUE;
        }
    }
    return FALSE;
}
