/*
Copyright (C) 2020 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2017 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif

#ifndef _WIN32_IE
#define _WIN32_IE 0x0500 // should be at least ie 5
#endif

// Enable Visual Style
#if defined _AMD64
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='amd64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#elif defined _WIN64
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='ia64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#else
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='x86' publicKeyToken='6595b64144ccf1df' language='*'\"")
#endif


#include <Windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

#include <Commctrl.h>
#pragma comment(lib,"Comctl32.lib")

#include "..\Tools\File\StdFileOperations.h"
#include "..\Tools\Exception\GenericTryCatchWithExceptionReport.h"

#include "resource.h"
#include "CommandLineParsing.h"
#include "DialogDesktopIconsPositions.h"

#ifdef _DEBUG
    #include "..\Tools\MemoryCheck\VisualStudioMemoryLeaksChecker.h"
#endif

int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine,
                   int nCmdShow
                   )
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);

#ifdef _DEBUG
    CVisualStudioMemoryLeaksChecker::Initialize();
#endif

    BOOL bCommandLineMode;
    CCommandLineInfo CommandLineInfo;

    CGenericTryCatchWithExceptionReport_TRY

    // enable Xp style
    InitCommonControls();
    ::OleInitialize(NULL);
    if (!ParseCommandLine(&bCommandLineMode,&CommandLineInfo))
        goto CleanUp;

#ifdef _DEBUG
    CVisualStudioMemoryLeaksChecker::Initialize();
#endif
    {
        CDialogDesktopIconsPositions DialogDesktopIconsPositions;
        if (bCommandLineMode)
        {
            switch(CommandLineInfo.Operation)
            {
            case CCommandLineInfo::Operation_RESTORE:
                DialogDesktopIconsPositions.Restore(CommandLineInfo.pFilePath,CommandLineInfo.bVerbose);
                break;
            case CCommandLineInfo::Operation_SAVE:
                DialogDesktopIconsPositions.Save(CommandLineInfo.pFilePath,CommandLineInfo.bVerbose);
                break;
            }
        }
        else
            DialogDesktopIconsPositions.Show(hInstance,NULL,IDD_DIALOG_DESKTOP_ICON_POSITION,IDI_ICON_APP);
    }
    
CleanUp:
    ::OleUninitialize();

    CGenericTryCatchWithExceptionReport_CATCH

	return 0;
}



