/*
Copyright (C) 2017 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2017 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manages donations
//-----------------------------------------------------------------------------

#include "Donations.h"
#include <stdio.h>

BOOL CDonations::IsComputerUsingEuroCurrency()
{
    BOOL bEuroCurrency=FALSE;
    HKEY hKey;
    wchar_t szCurrency[2];
    DWORD Size=2*sizeof(wchar_t);
    memset(szCurrency,0,Size);

    // check that HKEY_CURRENT_USER\Control Panel\International\sCurrency contains the euro symbol
    // retrieve it in unicode to be quite sure of the money symbol
    if (RegOpenKeyEx(HKEY_CURRENT_USER,_T("Control Panel\\International"),0,KEY_QUERY_VALUE,&hKey)==ERROR_SUCCESS)
    {
        // use unicode version only to make string compare
        if (RegQueryValueExW(hKey,L"sCurrency",NULL,NULL,(LPBYTE)szCurrency,&Size)==ERROR_SUCCESS)
        {
            // if (wcscmp(szCurrency,L"")==0)
            if (*szCurrency == 0x20AC) // avoid unicode encoding for euro symbol
                bEuroCurrency=TRUE;
        }
        // close open key
        RegCloseKey(hKey);
    }

    return bEuroCurrency;
}

BOOL CDonations::MakePaypalDonation(HWND hWndParent,TCHAR* ProjectName,TCHAR* ProjectWebPageLink)
{
    TCHAR szMsg[1024];

    TCHAR* Currency = TEXT("&currency_code=USD");

    // in case of euro monetary symbol
    if (CDonations::IsComputerUsingEuroCurrency())
        Currency = TEXT("&currency_code=EUR");

    _sntprintf(szMsg,_countof(szMsg),
                TEXT("https://www.paypal.com/cgi-bin/webscr?cmd=_xclick&business=jacquelin.potier@free.fr")
                TEXT("%s&lc=EN&country=US&item_name=Donation%%20for%%20%s&return=%s"),
                Currency,ProjectName,ProjectWebPageLink);

    // open donation web page
    if (ShellExecute(NULL,_T("open"),szMsg,NULL,NULL,SW_SHOWNORMAL)<(HINSTANCE)33)
    {
        // display error msg in case of failure
        ::MessageBox(hWndParent,
                    TEXT("Error Opening default browser.\r\nYou can make a donation going to http://jacquelin.potier.free.fr"),
                    TEXT("Error"),
                    MB_OK|MB_ICONERROR
                    );
        return FALSE;
    }
    return TRUE;
}