/*
Copyright (C) 2014-2020 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2014 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: get windows desktop work area for single or multiple monitors 
//         as ::SystemParametersInfo(SPI_GETWORKAREA,0,&Rect,0); returns only main monitor rect
//-----------------------------------------------------------------------------

#include "DesktopWorkArea.h"

BOOL CALLBACK CDesktopWorkArea::GetWorkAreaCallback(
                                                    HMONITOR hMonitor,  // handle to display monitor
                                                    HDC hdcMonitor,     // handle to monitor DC
                                                    LPRECT lprcMonitor, // monitor intersection rectangle
                                                    LPARAM lpData       // data
                                                    )
{
    UNREFERENCED_PARAMETER(hdcMonitor);
    UNREFERENCED_PARAMETER(lprcMonitor);

    GET_WORK_AREA_CALLBACK_PARAM* pEnumDisplayParam;
    MONITORINFOEX MonitorInfo;
    MonitorInfo.cbSize = sizeof(MonitorInfo);
    ::GetMonitorInfo(hMonitor,&MonitorInfo);

    pEnumDisplayParam = (GET_WORK_AREA_CALLBACK_PARAM*)lpData;
    RECT* pUsedRect;
    if (pEnumDisplayParam->GetMonitorRectInsteadOfWorkArea)
        pUsedRect = &MonitorInfo.rcMonitor;
    else
        pUsedRect = &MonitorInfo.rcWork;

    // to get display bounding
    // ::UnionRect((LPRECT)pBoundingRect, (LPRECT)pBoundingRect, lprcMonitor);
    ::UnionRect(&pEnumDisplayParam->AllDisplaysWorkAreaRect, &pEnumDisplayParam->AllDisplaysWorkAreaRect, pUsedRect);
    if (!pEnumDisplayParam->PrimaryDisplayInfoFilled)
    {
        pEnumDisplayParam->PrimaryDisplayWorkAreaRect = *pUsedRect;
        pEnumDisplayParam->PrimaryDisplayInfoFilled = TRUE;
    }
    return TRUE;
}


BOOL CDesktopWorkArea::GetWorkArea(OUT RECT* pWorkAreaRect,BOOL bPrimaryDisplayMonitorOnly,BOOL bGetFullMonitorRectInsteadOfWorkArea)
{
    // Notice:
    // ::SystemParametersInfo(SPI_GETWORKAREA,0,pWorkAreaRect,0); returns only main monitor rect
    // ::GetSystemMetrics(SM_CXVIRTUALSCREEN) and ::GetSystemMetrics(SM_CYVIRTUALSCREEN) don't provide work area
/*
    if (bPrimaryDisplayMonitorOnly)
        ::SystemParametersInfo(SPI_GETWORKAREA,0,pWorkAreaRect,0);
    else
    {
        GET_WORK_AREA_CALLBACK_PARAM EnumDisplayParam;
        memset(&EnumDisplayParam,0,sizeof(EnumDisplayParam));
        // EnumDisplayParam.PrimaryDisplayInfoFilled = FALSE; // not required until FALSE == 0
        BOOL bRet = ::EnumDisplayMonitors(
                                            NULL, // handle to display DC
                                            NULL, // clipping rectangle  
                                            CDesktopWorkArea::GetWorkAreaCallback, // callback function
                                            (LPARAM)&EnumDisplayParam // data for callback function 
                                         );
        *pWorkAreaRect = EnumDisplayParam.AllDisplaysWorkAreaRect;
        return bRet;
    }
*/
    RECT PrimaryDisplayWorkAreaRect;
    RECT AllDisplaysWorkAreaRect;
    BOOL bRet = CDesktopWorkArea::GetWorkAreaEx(&PrimaryDisplayWorkAreaRect,&AllDisplaysWorkAreaRect,bGetFullMonitorRectInsteadOfWorkArea);
    if (bPrimaryDisplayMonitorOnly)
        *pWorkAreaRect = PrimaryDisplayWorkAreaRect;
    else
        *pWorkAreaRect = AllDisplaysWorkAreaRect;
    return bRet;
}

BOOL CDesktopWorkArea::GetWorkAreaEx(OUT RECT* pPrimaryDisplayWorkAreaRect,OUT RECT* pAllDisplaysWorkAreaRect,BOOL bGetFullMonitorRectInsteadOfWorkArea)
{
    GET_WORK_AREA_CALLBACK_PARAM EnumDisplayParam;
    memset(&EnumDisplayParam,0,sizeof(EnumDisplayParam));
    // EnumDisplayParam.PrimaryDisplayInfoFilled = FALSE; // not required until FALSE == 0
    EnumDisplayParam.GetMonitorRectInsteadOfWorkArea=bGetFullMonitorRectInsteadOfWorkArea;
    BOOL bRet = ::EnumDisplayMonitors(
                                        NULL, // handle to display DC
                                        NULL, // clipping rectangle  
                                        CDesktopWorkArea::GetWorkAreaCallback, // callback function
                                        (LPARAM)&EnumDisplayParam // data for callback function 
                                        );
    *pPrimaryDisplayWorkAreaRect = EnumDisplayParam.PrimaryDisplayWorkAreaRect;
    *pAllDisplaysWorkAreaRect = EnumDisplayParam.AllDisplaysWorkAreaRect;
    return bRet;
}


BOOL CALLBACK CDesktopWorkArea::GetNumberOfDisplayMonitorCallback(
                                                                    HMONITOR hMonitor,  // handle to display monitor
                                                                    HDC hdcMonitor,     // handle to monitor DC
                                                                    LPRECT lprcMonitor, // monitor intersection rectangle
                                                                    LPARAM lpData       // data
                                                                    )
{
    UNREFERENCED_PARAMETER(hMonitor);
    UNREFERENCED_PARAMETER(hdcMonitor);
    UNREFERENCED_PARAMETER(lprcMonitor);
    DWORD* pNbMonitors = (DWORD*)lpData;
    (*pNbMonitors)++;
    return TRUE;
}

DWORD CDesktopWorkArea::GetNumberOfDisplayMonitors()
{
    DWORD NbMonitors = 0;
    ::EnumDisplayMonitors(
                            NULL, // handle to display DC
                            NULL, // clipping rectangle  
                            CDesktopWorkArea::GetNumberOfDisplayMonitorCallback, // callback function
                            (LPARAM)&NbMonitors // data for callback function 
                            );
    return NbMonitors;
}



BOOL CALLBACK CDesktopWorkArea::GetWorkAreaForMonitorCallback(
                                                                HMONITOR hMonitor,  // handle to display monitor
                                                                HDC hdcMonitor,     // handle to monitor DC
                                                                LPRECT lprcMonitor, // monitor intersection rectangle
                                                                LPARAM lpData       // data
                                                                )
{
    UNREFERENCED_PARAMETER(hdcMonitor);
    UNREFERENCED_PARAMETER(lprcMonitor);

    GET_WORK_AREA_FOR_MONITOR_CALLBACK_PARAM* pEnumDisplayParam;
    MONITORINFOEX MonitorInfo;
    MonitorInfo.cbSize = sizeof(MonitorInfo);
    ::GetMonitorInfo(hMonitor,&MonitorInfo);

    pEnumDisplayParam = (GET_WORK_AREA_FOR_MONITOR_CALLBACK_PARAM*)lpData;
    RECT* pUsedRect;
    if (pEnumDisplayParam->GetMonitorRectInsteadOfWorkArea)
        pUsedRect = &MonitorInfo.rcMonitor;
    else
        pUsedRect = &MonitorInfo.rcWork;


    BOOL bFound;
    switch(pEnumDisplayParam->Way)
    {
        default:
        case GetWorkAreaForMonitorWay_FROM_INDEX:
            bFound = (pEnumDisplayParam->WantedMonitorIndex == pEnumDisplayParam->CurrentMonitorIndex);
            break;
        case GetWorkAreaForMonitorWay_FROM_POINT:
            bFound = ::PtInRect(pUsedRect,pEnumDisplayParam->Point);
            break;
    }

    if (!bFound)
    {
        pEnumDisplayParam->CurrentMonitorIndex++;
        // continue the enumeration;
        return TRUE;
    }

    pEnumDisplayParam->DisplayInfoFilled = TRUE;
    pEnumDisplayParam->DisplayWorkAreaRect = *pUsedRect;

    // stop the enumeration
    return FALSE;
}

BOOL CDesktopWorkArea::GetWorkAreaForMonitor(IN DWORD ZeroBasedMonitorIndex,OUT RECT* pWorkAreaRect,BOOL bGetFullMonitorRectInsteadOfWorkArea)
{
    if (ZeroBasedMonitorIndex>=CDesktopWorkArea::GetNumberOfDisplayMonitors())
    {
        // return primary monitor work area
        CDesktopWorkArea::GetWorkAreaForMonitor(0,pWorkAreaRect);
        return FALSE;
    }

    GET_WORK_AREA_FOR_MONITOR_CALLBACK_PARAM Param={0};
    Param.Way=GetWorkAreaForMonitorWay_FROM_INDEX;
    Param.WantedMonitorIndex = ZeroBasedMonitorIndex;
    Param.CurrentMonitorIndex = 0;
    Param.DisplayInfoFilled = FALSE;
    Param.GetMonitorRectInsteadOfWorkArea=bGetFullMonitorRectInsteadOfWorkArea;

    ::EnumDisplayMonitors(
                            NULL, // handle to display DC
                            NULL, // clipping rectangle  
                            CDesktopWorkArea::GetWorkAreaForMonitorCallback, // callback function
                            (LPARAM)&Param // data for callback function 
                            );
    *pWorkAreaRect = Param.DisplayWorkAreaRect;

    return Param.DisplayInfoFilled;
}

BOOL CDesktopWorkArea::GetWorkAreaForMonitorFromPoint(IN POINT* pPoint,OUT DWORD* pMonitorIndex,OUT RECT* pWorkAreaRect,BOOL bGetFullMonitorRectInsteadOfWorkArea)
{
    GET_WORK_AREA_FOR_MONITOR_CALLBACK_PARAM Param={0};
    Param.Way=GetWorkAreaForMonitorWay_FROM_POINT;
    Param.Point = *pPoint;
    Param.CurrentMonitorIndex = 0;
    Param.DisplayInfoFilled = FALSE;
    Param.GetMonitorRectInsteadOfWorkArea=bGetFullMonitorRectInsteadOfWorkArea;

    ::EnumDisplayMonitors(
                            NULL, // handle to display DC
                            NULL, // clipping rectangle  
                            CDesktopWorkArea::GetWorkAreaForMonitorCallback, // callback function
                            (LPARAM)&Param // data for callback function 
                            );
    *pWorkAreaRect = Param.DisplayWorkAreaRect;

    if (pMonitorIndex)
        *pMonitorIndex = Param.CurrentMonitorIndex;

    return Param.DisplayInfoFilled;
}