/*
Copyright (C) 2020 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2017 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/////////////////////////////////////////////////////
// Usage
/////////////////////////////////////////////////////
// CVisualStudioMemoryLeaksChecker::Initialize();
//
// CVisualStudioMemoryLeaksChecker::ShowLeaks(); // optional, should not be called explicitly to avoid to report global object
//
// // to check memory difference between 2 points
// _CrtMemState MemStateRef;
// CVisualStudioMemoryLeaksChecker::GetCurrentMemoryState(&MemStateRef);
// CVisualStudioMemoryLeaksChecker::CompareReferenceMemoryStateWithCurrentMemoryState(&MemStateRef);
/////////////////////////////////////////////////////

#ifdef _DEBUG 

#pragma once

#include <windows.h>
#include <crtdbg.h>

#ifdef _DEBUG
    // may to see later _CrtSetAllocHook
    // https://www.codeproject.com/Articles/9815/Visual-Leak-Detector-Enhanced-Memory-Leak-Detectio
    // https://github.com/oneiric/vld
#endif

class CVisualStudioMemoryLeaksChecker
{
public:
    static FORCEINLINE int Initialize()
    {
        int RetValue = _CrtSetDbgFlag (   _CRTDBG_ALLOC_MEM_DF 
                                        | _CRTDBG_LEAK_CHECK_DF 
                                        // | _CRTDBG_DELAY_FREE_MEM_DF
                                       );

        _CrtSetReportMode( _CRT_ERROR, _CRTDBG_MODE_DEBUG );
        
        return RetValue;
    }
    static FORCEINLINE int ShowLeaks() 
    { 
        
        return _CrtDumpMemoryLeaks(); 
    }


    static FORCEINLINE void GetCurrentMemoryState(OUT _CrtMemState* pMemState)
    {
        _CrtMemCheckpoint(pMemState);
    }

    static FORCEINLINE void CompareReferenceMemoryStateWithCurrentMemoryState(IN _CrtMemState* pReferenceMemoryState)
    {
        _CrtMemState s2;
        _CrtMemState s3;
        _CrtMemCheckpoint(&s2);

        if (_CrtMemDifference(&s3, pReferenceMemoryState, &s2))
            _CrtMemDumpStatistics(&s3); // no memory difference found; thus no leak
    }
};

#endif  // _DEBUG 
